/*
 *     This file is part of MediLog.
 *
 *     MediLog is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Affero General Public License as published by
 *     the Free Software Foundation.
 *
 *     MediLog is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU Affero General Public License for more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with MediLog.  If not, see <http://www.gnu.org/licenses/>.
 *
 *     Copyright (c) 2018 - 2025 by Zell-MBC.com
 */

/*
 */

package com.zell_mbc.medilog.fluid

import android.app.Application
import com.zell_mbc.medilog.MainActivity.Companion.DAY_FORMAT
import com.zell_mbc.medilog.Tabs
import com.zell_mbc.medilog.MainActivity.Companion.activeProfileId
import com.zell_mbc.medilog.MainActivity.Companion.viewModel
import com.zell_mbc.medilog.R
import com.zell_mbc.medilog.R.string
import com.zell_mbc.medilog.data.Data
import com.zell_mbc.medilog.data.DataViewModel
import com.zell_mbc.medilog.preferences.SettingsActivity
import java.text.SimpleDateFormat
import java.util.Date

class FluidViewModel(application: Application): DataViewModel(application, Tabs.FLUID) {
    override val filterStartPref = "FLUIDFILTERSTART"
    override val filterEndPref = "FLUIDFILTEREND"
    override val timeFilterModePref =  "FLUID_FILTER_MODE"
    override val rollingFilterValuePref = "FLUID_ROLLING_FILTER_VALUE"
    override val rollingFilterTimeframePref = "FLUID_ROLLING_FILTER_TIMEFRAME"
    override val tagFilterPref = "FLUID_TAG_FILTER"

    override var itemName = app.getString(string.fluid)

    override var blendInItems = preferences.getBoolean(SettingsActivity.KEY_PREF_FLUID_BLENDINITEMS, app.getString(string.BLENDINITEMS_DEFAULT).toBoolean())
    var tagsEnabled = false

    fun tagCount(tagId: Int = 0, filtered: Boolean = true): Int {
        var string = "SELECT count (*) FROM data WHERE profile_id = $activeProfileId AND type = $dataType AND tags "
        string += if (tagId == 0) "= ''" else "LIKE '%$tagId%'"

        if (filtered) {
            getFilter()
            if ((filterStart != 0L) && (filterEnd != 0L)) string += " AND timestamp >= $filterStart AND timestamp <= $filterEnd"
            if ((filterStart == 0L) && (filterEnd != 0L)) string += " AND timestamp <= $filterEnd"
            if ((filterStart != 0L) && (filterEnd == 0L)) string += " AND timestamp >= $filterStart"
        }
        return getInt(string)
    }


    fun getTodayTagSum(tagId: Int): Int {
        val today = SimpleDateFormat(DAY_FORMAT).format(Date().time)
        var string = "SELECT SUM(CAST(value1 as int)) as value1 from data WHERE profile_id = $activeProfileId and type=$dataType and value2 = '$today' AND tags "
        string += if (tagId == 0) "LIKE ''" else "LIKE '%$tagId%'"

        return getInt(string)
    }

    // TagId = -1 to ignore tags, = 0 to filter empty tags
    fun getDays(filtered: Boolean, order: String, tagId: Int = -1): MutableList<Data> {
        // Spell out all fields as otherwise the result can't be matched to item = activity crash
        // Make sure to call out all fields explicitly as otherwise the sql statement will crash!!
        //var string = "SELECT _id, timestamp, comment, type, SUM(CAST(value1 as int)) AS value1, value2, value3, value4, attachment, profile_id, category_id, tags FROM data WHERE "
        //var string = "SELECT _id, (strftime('%s', value2 || ' 23:59:59') * 1000) + 999 AS timestamp, comment, type, SUM(CAST(value1 as int)) AS value1, value2, value3, value4, attachment, profile_id, category_id, tags FROM data WHERE "
        var string = "SELECT _id, (strftime('%s', value2) * 1000) + 999 AS timestamp, comment, type, SUM(CAST(value1 as int)) AS value1, value2, value3, value4, attachment, profile_id, category_id, tags FROM data WHERE "
        string += if (blendInItems) " (type = $dataType OR type = ${Tabs.DIARY})"
        else "type=$dataType"

        string += " AND profile_id = $activeProfileId"

        if (tagId >= 0) {
            string += " AND tags " + if (tagId == 0) "= ''" else "LIKE '%$tagId%'"
        }

        if (filtered) {
            getFilter()
            if ((filterStart != 0L) && (filterEnd != 0L)) string += " AND timestamp >= $filterStart AND timestamp <= $filterEnd"
            if ((filterStart == 0L) && (filterEnd != 0L)) string += " AND timestamp <= $filterEnd"
            if ((filterStart != 0L) && (filterEnd == 0L)) string += " AND timestamp >= $filterStart"
        }
        // GROUP BY day
        string += " GROUP BY value2 ORDER BY value2 $order"
        return getDataList(string)
    }

    override fun checkValues(): String {
        // Check empty variables
        if (field1.value.isEmpty()) {
            return app.getString(string.valuesMissing)
        }

        field1.value = field1.value.replace(decimalSeparator, '.')

        //val pattern = "\\d{1," + viewModel.field1.format.length.toString() + "}$"
        val pattern = "\\d{1," + viewModel.field1.format.length.toString() + "}"
        try {
            if (!Regex(pattern).matches(field1.value)) return app.getString(string.invalidFormat) + "\n" + app.getString(string.configuredFormat, "\"" + field1.format.replace('.', decimalSeparator) + "\"")
        }
        catch (_: Exception) {
            return app.getString(string.unknownError) + "\nRegex: $pattern\nValue: " + viewModel.field1.value +"\nValue: " + viewModel.field1.format
        }

        return ""
    }

    override fun setField1Parameters() {
        val customName = preferences.getString(SettingsActivity.KEY_PREF_FLUID_CUSTOM_NAME, "").toString()
        if (customName.isEmpty()) {
            customTab = false
            field1.name = app.getString(string.fluid)
            field1.unit = preferences.getString(SettingsActivity.KEY_PREF_FLUID_UNIT, app.getString(string.FLUID_UNIT_DEFAULT)).toString()
            field1.format = app.getString(string.FLUID_THRESHOLD_DEFAULT).replace(Regex("."), "9")
        } else {
            customTab = true
            field1.name = customName
            field1.unit = preferences.getString(SettingsActivity.KEY_PREF_FLUID_CUSTOM_UNIT, "").toString()
            field1.format = preferences.getString(SettingsActivity.KEY_PREF_FLUID_CUSTOM_DIGITS, app.getString(string.FLUID_THRESHOLD_DEFAULT).replace(Regex("."), "9")).toString()
        }
        super.setFieldParameters(1)
    }
}