/*
 *     This file is part of MediLog.
 *
 *     MediLog is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Affero General Public License as published by
 *     the Free Software Foundation.
 *
 *     MediLog is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU Affero General Public License for more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with MediLog.  If not, see <http://www.gnu.org/licenses/>.
 *
 *     Copyright (c) 2018 - 2025 by Zell-MBC.com
 */

package com.zell_mbc.medilog.onboarding

import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Text
import androidx.compose.material3.Surface
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp


data class OnboardingModel(
    val imageRes: Int,
    val title: String,
    val description: String
)

@Composable
fun OnboardingItem(page: OnboardingModel) {
    Column(
        modifier = Modifier.fillMaxSize(),
           horizontalAlignment = Alignment.CenterHorizontally,
           verticalArrangement = Arrangement.Center
    ) {
        Surface(shape = RoundedCornerShape(16.dp)) {
            Image(painter = painterResource(id = page.imageRes), contentDescription = null,
               modifier = Modifier.height(350.dp).width(350.dp).padding(bottom = 20.dp)
            )
        }
        Text(text = page.title, style = TextStyle(fontSize = 24.sp, fontWeight = FontWeight.Bold, color = Color(0xFF232B39), textAlign = TextAlign.Center))
        Text(text = page.description,
             modifier = Modifier.padding(horizontal = 16.dp, vertical = 10.dp),
             style = TextStyle(fontSize = 14.sp, fontWeight = FontWeight.W400, color = Color(0xFF707784), textAlign = TextAlign.Center)
        )
    }

}
