/*
 *     This file is part of MediLog.
 *
 *     MediLog is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Affero General Public License as published by
 *     the Free Software Foundation.
 *
 *     MediLog is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU Affero General Public License for more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with MediLog.  If not, see <http://www.gnu.org/licenses/>.
 *
 *     Copyright (c) 2018 - 2025 by Zell-MBC.com
 */

package com.zell_mbc.medilog.preferences

import android.os.Bundle
import android.view.View
import androidx.core.view.ViewCompat
import androidx.core.view.WindowInsetsCompat
import androidx.preference.Preference
import androidx.preference.PreferenceFragmentCompat
import androidx.preference.PreferenceManager
import com.zell_mbc.medilog.MainActivity
import com.zell_mbc.medilog.Tabs.GLUCOSE
import com.zell_mbc.medilog.Tabs.TEMPERATURE
import com.zell_mbc.medilog.R
import com.zell_mbc.medilog.R.string

class SettingsFragment : PreferenceFragmentCompat() {
     override fun onCreatePreferences(savedInstanceState: Bundle?, rootKey: String?) {
        setPreferencesFromResource(R.xml.preferences, rootKey)

         //val label = if (preferences.getString(SettingsActivity.KEY_PREF_GLUCOSE_UNIT,"") == getString(string.COAGULATION_UNIT_INR)) getString(string.coagulation) else getString(string.glucose)

         // Hide preferences for unused tabs
         for (tab in MainActivity.availableTabs) {
             val id = tab.id.toString()
             val pref: Preference? = preferenceManager.findPreference("id_$id")
             pref?.isVisible = tab.active
            if (tab.active) {// && pref?.title == getString(string.glucose))
                val customTitle = getName(tab.id)
                if (customTitle.isNotEmpty()) pref?.title = customTitle
            }
         }
     }

    fun getName(id: Int): String {
        var tabName = ""
        when (id) {
            GLUCOSE -> {
                val label = PreferenceManager.getDefaultSharedPreferences(requireActivity()).getString(SettingsActivity.KEY_PREF_GLUCOSE_CUSTOM_NAME, "")
                tabName = if (label.isNullOrEmpty()) getString(string.glucose) else label
            }
            TEMPERATURE -> {
                val label = PreferenceManager.getDefaultSharedPreferences(requireActivity()).getString(SettingsActivity.KEY_PREF_TEMPERATURE_CUSTOM_NAME, "")
                tabName = if (label.isNullOrEmpty()) getString(string.temperature) else label
            }
        }
        return tabName
    }
}