/*
 *     This file is part of MediLog.
 *
 *     MediLog is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Affero General Public License as published by
 *     the Free Software Foundation.
 *
 *     MediLog is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU Affero General Public License for more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with MediLog.  If not, see <http://www.gnu.org/licenses/>.
 *
 *     Copyright (c) 2018 - 2025 by Zell-MBC.com
 */

/*
 */

package com.zell_mbc.medilog.temperature

import android.app.Application
import com.zell_mbc.medilog.R.string
import com.zell_mbc.medilog.Tabs
import com.zell_mbc.medilog.data.DataViewModel
import com.zell_mbc.medilog.preferences.SettingsActivity

class TemperatureViewModel(application: Application): DataViewModel(application, Tabs.TEMPERATURE) {
    override val filterStartPref = "TEMPERATUREFILTERSTART"
    override val filterEndPref = "TEMPERATUREFILTEREND"
    override val timeFilterModePref = "TEMPERATURE_FILTER_MODE"
    override val rollingFilterValuePref = "TEMPERATURE_ROLLING_FILTER_VALUE"
    override val rollingFilterTimeframePref = "TEMPERATURE_ROLLING_FILTER_TIMEFRAME"
    override val tagFilterPref = "TEMPERATURE_TAG_FILTER"

    override var itemName = app.getString(string.temperature)
    override var blendInItems = preferences.getBoolean(SettingsActivity.KEY_PREF_TEMPERATURE_BLENDINITEMS, false)

    override fun checkValues(): String { return "" }
/*        // Check empty variables
        if (field1.value.isEmpty()) {
            return app.getString(string.valuesMissing)
        }

        field1.value = field1.value.replace(decimalSeparator, '.')

        // Check if entry matches defined format
        var pattern = ""
        if (viewModel.field1.isInteger) pattern = "\\d{1," + viewModel.field1.format.length.toString() + "}"
        else {
            val d = field1.format.indexOf('.')
            val e = viewModel.field1.format.substring(d).length -1
            pattern = "\\d{1," + d + "}.\\d{1," + e + "}"
        }
        if (!Regex(pattern).matches(field1.value)) return app.getString(string.invalidFormat) + "\n" + app.getString(string.configuredFormat, field1.format)

        return ""
    }*/

    // Specific parameters, applicable to just this field
    override fun setField1Parameters() {
        val customName = "" + preferences.getString(SettingsActivity.KEY_PREF_TEMPERATURE_CUSTOM_NAME, "")
        if (customName.isEmpty()) {
            customTab = false
            field1.name = app.getString(string.temperature)
            field1.unit = "" + preferences.getString(SettingsActivity.KEY_PREF_TEMPERATURE_UNIT, app.getString(string.TEMPERATURE_UNIT_DEFAULT)).toString()
            val celsius = (field1.unit == app.getString(string.TEMPERATURE_CELSIUS))
            field1.format = (if (celsius) "" else "0") + app.getString(string.temperatureHint)
        } else {
            customTab = true
            field1.name = customName
            field1.unit = preferences.getString(SettingsActivity.KEY_PREF_TEMPERATURE_CUSTOM_UNIT, "").toString()
            field1.format = preferences.getString(SettingsActivity.KEY_PREF_TEMPERATURE_CUSTOM_DIGITS, "9.9").toString()
        }
        setFieldParameters(1)  // Generic  parameters, applicable to all fields
    }
}

