/*
 *     This file is part of MediLog.
 *
 *     MediLog is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Affero General Public License as published by
 *     the Free Software Foundation.
 *
 *     MediLog is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU Affero General Public License for more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with MediLog.  If not, see <http://www.gnu.org/licenses/>.
 *
 *     Copyright (c) 2018 - 2025 by Zell-MBC.com
 */

/*
 */

package com.zell_mbc.medilog.weight

import android.app.Application
import com.zell_mbc.medilog.Tabs
import com.zell_mbc.medilog.R.string
import com.zell_mbc.medilog.data.DataViewModel
import com.zell_mbc.medilog.preferences.SettingsActivity

class WeightViewModel(application: Application): DataViewModel(application, Tabs.WEIGHT) {
    override val filterStartPref = "WEIGHTFILTERSTART"
    override val filterEndPref = "WEIGHTFILTEREND"
    override val timeFilterModePref =  "WEIGHT_FILTER_MODE"
    override val rollingFilterValuePref = "WEIGHT_ROLLING_FILTER_VALUE"
    override val rollingFilterTimeframePref = "WEIGHT_ROLLING_FILTER_TIMEFRAME"
    override val tagFilterPref = "WEIGHT_TAG_FILTER"

    override var blendInItems = preferences.getBoolean(SettingsActivity.KEY_PREF_WEIGHT_BLENDINITEMS, app.getString(string.BLENDINITEMS_DEFAULT).toBoolean())

    override var itemName = app.getString(string.weight) // Todo: Get rid of this once field 1 is custom for all

    override fun checkValues(): String { return "" }

    // Specific parameters, applicable to just this field
    override fun setField1Parameters() {
        val customName = "" + preferences.getString(SettingsActivity.KEY_PREF_WEIGHT_CUSTOM_NAME, "")
        if (customName.isEmpty()) {
            customTab = false
            field1.name = app.getString(string.weight)
            field1.unit = "" + preferences.getString(SettingsActivity.KEY_PREF_WEIGHT_UNIT, app.getString(string.WEIGHT_UNIT_DEFAULT)).toString()
            field1.format = app.getString(string.weightHint)
        } else {
            customTab = true
            field1.name = customName
            field1.unit = preferences.getString(SettingsActivity.KEY_PREF_WEIGHT_CUSTOM_UNIT, "").toString()
            field1.format = preferences.getString(SettingsActivity.KEY_PREF_WEIGHT_CUSTOM_DIGITS, "99.9").toString()
        }
        setFieldParameters(1)  // Generic  parameters, applicable to all fields
    }
}